package com.cloud.kicc.common.datasource.util;

import com.baomidou.dynamic.datasource.DynamicRoutingDataSource;
import com.baomidou.dynamic.datasource.creator.DefaultDataSourceCreator;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DataSourceProperty;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DynamicDataSourceProperties;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.cloud.kicc.common.datasource.dynamic.DynamicDataSource;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.map.LRUMap;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;


/**
 *<p>
 * 动态数据源核心处理工具
 *</p>
 *
 * @Author: wangxiang4
 * @Since: 2023/7/3
 */
@Setter
@Slf4j
@Component
public class DynamicDataSourceUtil {

	public static DynamicRoutingDataSource dynamicRoutingDataSource;
	public static DynamicDataSourceProperties dynamicDataSourceProperties;
	private static DefaultDataSourceCreator defaultDataSourceCreator;
	private static int MAX_DATASOURCE_COUNT = 300;
	// 最多保存三百个数据源,按使用率淘汰
	private static final LRUMap<String, DynamicDataSource> linksProperties = new LRUMap(MAX_DATASOURCE_COUNT);

	public DynamicDataSourceUtil(DataSource  dynamicRoutingDataSource,
								 DynamicDataSourceProperties dynamicDataSourceProperties,
								 DefaultDataSourceCreator defaultDataSourceCreator) {
		DynamicDataSourceUtil.dynamicRoutingDataSource = (DynamicRoutingDataSource) dynamicRoutingDataSource;
		DynamicDataSourceUtil.dynamicDataSourceProperties = dynamicDataSourceProperties;
		DynamicDataSourceUtil.defaultDataSourceCreator = defaultDataSourceCreator;
	}

	/**
	 * 创建并切换至远程数据源
	 * @param dynamicDataSource 切换数据源
	 */
	public static void switchToDataSource(DynamicDataSource dynamicDataSource) {
		String dbKey = dynamicDataSource.getId();
		String removeKey = null;
		boolean insert = true;
		if (dynamicRoutingDataSource.getDataSources().containsKey(dynamicDataSource.getId())) {
			synchronized (linksProperties) {
				if (linksProperties.get(dbKey).equals(dynamicDataSource)) {
					insert = false;
				}
			}
		}
		if (insert) {
			// 创建数据源配置
			DataSourceProperty dataSourceProperty = new DataSourceProperty();
			// 拷贝数据源配置
			BeanUtils.copyProperties(dynamicDataSource, dataSourceProperty);
			// 创建动态数据源
			DataSource dataSource = defaultDataSourceCreator.createDataSource(dataSourceProperty);
			// 添加最新数据源
			dynamicRoutingDataSource.addDataSource(dbKey, dataSource);
			synchronized (linksProperties) {
				if (linksProperties.size() == MAX_DATASOURCE_COUNT) {
					removeKey = linksProperties.firstKey();
				}
				linksProperties.put(dbKey, dynamicDataSource);
			}
		}
		// 切换数据源
		DynamicDataSourceContextHolder.push(dbKey);
		if (removeKey != null) {
			try {
				dynamicRoutingDataSource.removeDataSource(removeKey);
			} catch (Exception e) {
				log.error("移除数据源失败：{}", e.getMessage());
			}
		}
	}

	/**
	 * 移除当前设置的远程数据源，清除上次清除之后切换的所有数据源
	 * 需要先调用 switchToDataSource切换数据源
	 */
	public static void clearSwitchDataSource() {
		DynamicDataSourceContextHolder.poll();
	}

	/**
	 * 获取当前数据源的数据链接(切库后的)
	 * 用完之后一定要关闭
	 * @return
	 * @throws SQLException
	 */
	public static Connection getCurrentConnection() throws SQLException {
		return dynamicRoutingDataSource.getConnection();
	}

	public static boolean containsLink(String key) {
		return linksProperties.containsKey(key);
	}

}
