package com.cloud.kicc.common.security.exp;

import cn.hutool.core.util.ClassUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.json.JSONUtil;
import com.cloud.kicc.common.core.constant.SecurityConstants;
import com.cloud.kicc.common.core.jackson.KiccJavaTimeModule;
import com.cloud.kicc.common.data.entity.CasUser;
import com.cloud.kicc.common.security.exception.UnConfiguredUserDataException;
import com.cloud.kicc.common.security.override.jackson2.SimpleGrantedAuthorityMixin;
import com.cloud.kicc.common.security.properties.CasProperties;
import com.cloud.kicc.common.security.template.UserProviderTemplate;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.netty.util.internal.StringUtil;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.common.exceptions.InvalidTokenException;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.OAuth2Request;
import org.springframework.security.oauth2.provider.token.ResourceServerTokenServices;
import org.springframework.security.oauth2.provider.token.TokenStore;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 *<p>
 * 本地资源服务器令牌服务
 *</p>
 *
 * @Author: wangxiang4
 * @Date: 2022/2/17
 */
@RequiredArgsConstructor
public class KiccLocalResourceServerTokenServices implements ResourceServerTokenServices {

	@Autowired
	private TokenStore tokenStore;

	@Autowired(required = false)
	private UserProviderTemplate userProviderTemplate;

	@Autowired
	private CasProperties casProperties;

	@Override
	@SneakyThrows
	public OAuth2Authentication loadAuthentication(String accessToken) throws AuthenticationException, InvalidTokenException {
		// 根据token加载身份验证
		OAuth2Authentication oAuth2Authentication = tokenStore.readAuthentication(accessToken);
		if (oAuth2Authentication == null) {
			return null;
		}
		OAuth2Request oAuth2Request = oAuth2Authentication.getOAuth2Request();
		// 检测是否是属于认证的CasUser实体用户
		if (!(oAuth2Authentication.getPrincipal() instanceof CasUser)) {
			return oAuth2Authentication;
		}
		CasUser casUser = (CasUser) oAuth2Authentication.getPrincipal();

		// 设置SSO子系统扩展用户信息
		if (casUser.getExPrincipals().get(casProperties.getIdentity()) == null) {
			Object user = userProviderTemplate.selectByCasUserId(casUser.getId());
			if (ObjectUtil.isEmpty(user)) {
				throw new UnConfiguredUserDataException("System user not found Contact your system administrator for configuration!", null);
			}

			String[] permissions = (String[])ReflectUtil.getMethodByName(user.getClass(), "getPermissions").invoke(user);
			List<SimpleGrantedAuthority> authorities = Arrays.stream(permissions)
					.map(SimpleGrantedAuthority::new)
					.collect(Collectors.toList());
			Object exUser = ReflectUtil.newInstance(ClassUtil.loadClass(casProperties.getUserClass()),
				casUser.getUsername(),
				StringUtil.EMPTY_STRING,
				casUser.isEnabled(),
				casUser.isAccountNonExpired(),
				casUser.isCredentialsNonExpired(),
				casUser.isAccountNonLocked(),
				authorities
			);
			BeanUtils.copyProperties(user, exUser);
			casUser.getExPrincipals().put(casProperties.getIdentity(), new ObjectMapper()
					.registerModule(new KiccJavaTimeModule())
					.setSerializationInclusion(JsonInclude.Include.NON_NULL)
					.writeValueAsString(exUser));
			tokenStore.storeAccessToken(tokenStore.getAccessToken(oAuth2Authentication), oAuth2Authentication);
		}

		// 覆盖casUser核心authorities
		String str = casUser.getExPrincipals().get(casProperties.getIdentity());
		if (!JSONUtil.isJson(str)) throw new UnConfiguredUserDataException("ExPrincipals not json strings!");
		Object exUser = new ObjectMapper()
				.registerModule(new KiccJavaTimeModule())
				.addMixIn(SimpleGrantedAuthority.class, SimpleGrantedAuthorityMixin.class)
				.readValue(str, Class.forName(casProperties.getUserClass()));
		Collection<? extends GrantedAuthority> authorities = (Collection<? extends GrantedAuthority>)ReflectUtil.getMethodByName(exUser.getClass(), "getAuthorities").invoke(exUser);
		CasUser exCasUser = new CasUser(
				casUser.getUsername(),
				SecurityConstants.MOCK_PASSWORD,
				casUser.isEnabled(),
				casUser.isAccountNonExpired(),
				casUser.isCredentialsNonExpired(),
				casUser.isAccountNonLocked(),
				authorities);
		BeanUtils.copyProperties(casUser, exCasUser);

		// 每次请求前都预先加载用户名密码身份验证令牌
		Authentication userAuthentication = new UsernamePasswordAuthenticationToken(exCasUser, "N/A", exCasUser.getAuthorities());
		OAuth2Authentication authentication = new OAuth2Authentication(oAuth2Request, userAuthentication);
		authentication.setAuthenticated(true);
		return authentication;
	}

	@Override
	public OAuth2AccessToken readAccessToken(String accessToken) {
		throw new UnsupportedOperationException("Not supported: read access token");
	}

}
