package com.cloud.kicc.common.security.util;

import cn.hutool.core.util.ClassUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.cloud.kicc.common.core.constant.CommonConstants;
import com.cloud.kicc.common.core.constant.SecurityConstants;
import com.cloud.kicc.common.core.jackson.KiccJavaTimeModule;
import com.cloud.kicc.common.core.util.SpringContextHolderUtil;
import com.cloud.kicc.common.data.entity.CasUser;
import com.cloud.kicc.common.security.exception.UnConfiguredUserDataException;
import com.cloud.kicc.common.security.override.jackson2.SimpleGrantedAuthorityMixin;
import com.cloud.kicc.common.security.properties.CasProperties;
import com.cloud.kicc.common.security.template.UserProviderTemplate;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import lombok.experimental.UtilityClass;
import org.springframework.beans.BeanUtils;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 *<p>
 * 安全工具类
 *</p>
 *
 * @Author: wangxiang4
 * @Date: 2022/2/17
 */
@UtilityClass
public class SecurityUtils {

	CasProperties casProperties = SpringContextHolderUtil.getBean(CasProperties.class);

	/**
	 * 获取Authentication
	 */
	public Authentication getAuthentication() {
		return SecurityContextHolder.getContext().getAuthentication();
	}

	/**
	 * 获取用户
	 */
	public CasUser getCasUser(Authentication authentication) {
		Object principal = authentication.getPrincipal();
		if (principal instanceof CasUser) {
			return (CasUser) principal;
		}
		return null;
	}

	/**
	 * 获取CAS用户
	 */
	public CasUser getCasUser() {
		Authentication authentication = getAuthentication();
		if (authentication == null) {
			return null;
		}
		return getCasUser(authentication);
	}

	/**
	 * 获取完整用户
	 * @param identity SSO系统身份
	 * @param valueType 自定义扩展用户
	 * @return T
	 */
	@SneakyThrows
	public <T extends CasUser> T getUser(String identity, Class<T> valueType) {
		CasUser casUser = getCasUser();
		if (casUser == null) return null;
		String str = casUser.getExPrincipals().get(identity);
		if (JSONUtil.isJson(str))
			return new ObjectMapper()
					.registerModule(new KiccJavaTimeModule())
					.addMixIn(SimpleGrantedAuthority.class, SimpleGrantedAuthorityMixin.class)
					.readValue(str, valueType);
		return null;
	}

	/**
	 * 获取SSO扩展用户
	 */
	public <T extends CasUser> T getUser() {
		return getUser(casProperties.getIdentity(), ClassUtil.loadClass(casProperties.getUserClass()));
	}

	/**
	 * 获取用户角色信息
	 * @return 角色集合
	 */
	public List<String> getRoles() {
		Authentication authentication = getAuthentication();
		Collection<? extends GrantedAuthority> authorities = authentication.getAuthorities();

		List<String> roleIds = new ArrayList<>();
		authorities.stream().filter(granted -> StrUtil.startWith(granted.getAuthority(), SecurityConstants.ROLE))
				.forEach(granted -> {
					String id = StrUtil.removePrefix(granted.getAuthority(), SecurityConstants.ROLE);
					roleIds.add(id);
				});
		return roleIds;
	}

	/**
	 * 对外开放接口临时登录会话
	 * @param userId 用户id
	 * @return User 用户对象
	 */
	@SneakyThrows
	public <T extends CasUser> T openInterfaceTemporaryLoginSession(String userId) {
		UserProviderTemplate userProviderTemplate = SpringContextHolderUtil.getBean(UserProviderTemplate.class);
		Object user = userProviderTemplate.selectByUserId(userId);
		if (ObjectUtil.isEmpty(user)) {
			throw new UnConfiguredUserDataException("System user not found Contact your system administrator for configuration!", null);
		}

		String[] permissions = (String[]) ReflectUtil.getMethodByName(user.getClass(), "getPermissions").invoke(user);
		List<SimpleGrantedAuthority> authorities = Arrays.stream(permissions)
				.map(SimpleGrantedAuthority::new)
				.collect(Collectors.toList());

		String username = (String) ReflectUtil.getMethodByName(user.getClass(), "getUsername").invoke(user);
		String password = (String) ReflectUtil.getMethodByName(user.getClass(), "getPassword").invoke(user);
		String status = (String) ReflectUtil.getMethodByName(user.getClass(), "getStatus").invoke(user);
		String[] roleIds = (String[]) ReflectUtil.getMethodByName(user.getClass(), "getRoleIds").invoke(user);
		String[] tenantIds = (String[]) ReflectUtil.getMethodByName(user.getClass(), "getTenantIds").invoke(user);

		CasUser casUser = new CasUser(
				username,
				SecurityConstants.BCRYPT + password,
				true,
				true,
				true,
				StrUtil.equals(status, CommonConstants.STATUS_NORMAL),
				authorities
		);
		BeanUtils.copyProperties(user, casUser);
		casUser.setRoleId(String.join(",", roleIds));
		casUser.setTenantId(String.join(",", tenantIds));
		Object exUser = ReflectUtil.newInstance(ClassUtil.loadClass(casProperties.getUserClass()),
				casUser.getUsername(),
				casUser.getPassword(),
				casUser.isEnabled(),
				casUser.isAccountNonExpired(),
				casUser.isCredentialsNonExpired(),
				casUser.isAccountNonLocked(),
				authorities
		);

		BeanUtils.copyProperties(user, exUser);
		// 设置扩展用户数据
		casUser.getExPrincipals().put(casProperties.getIdentity(), new ObjectMapper()
		.registerModule(new KiccJavaTimeModule())
		.setSerializationInclusion(JsonInclude.Include.NON_NULL)
		.writeValueAsString(exUser));
		Authentication authentication = new UsernamePasswordAuthenticationToken(casUser, "N/A", casUser.getAuthorities());
		SecurityContextHolder.getContext().setAuthentication(authentication);
		return (T) user;
	}

	public static void main(String[] args) {
		System.out.println(new BCryptPasswordEncoder().encode("123456"));
	}

}
